var Stations = (function($) {

	// Die Stationsdaten werden im LocalStorage gecacht und beim Appstart von doret gelesen
	var stationData = (function() {

			var value = window.localStorage['jet.app.stations'];
			return (value && JSON.parse(value)) || {};

		}()),

		// Timestamp des letzten Imports
		lastUpdate = stationData['lastUpdate'] || 0,

		// Filter (Zeige nur Stationen mit...)
		features = {
			lpg: false, // Autogas
			wash: false, // JET Wäsche
			spar_express: false, // spar express
			backshop: false // Backshop
		};

	// Hilfsfunktion um ein LatLng Objekt zu erstellen,
	// welches die Google Maps Api an manchen Stellen erwartet.
	// Wird anscheinend nicht mehr benötigt...
	function ll(lat, lng) {
		return {
			lat: function() { return lat; },
			lng: function() { return lng; }
		}
	}

	/**
	 * Gibt alle Stationen zurück, die sich in den "bounds" befinden.
	 *
	 * @var google.maps.LatLngBounds bounds
	 * @return array
	 */
	function findStationsInBounds(bounds) {

		var station_id,
			station, 
			ll,
			stations = [];

		for(station_id in stationData) {
			station = stationData[station_id];

			// Filter by Type (LPG, wash, spar_express, backshop)
			if(features.lpg && !station.lpg) continue;
			if(features.wash && station.wash != 'PRK') continue;
			if(features.spar_express && station.spar_express != '602') continue;
			if(features.backshop && !station.backshop) continue;

			if(bounds.contains(new google.maps.LatLng(station.lat, station.lng))) {
				stations.push(station);
			}

		}



		JET.trigger('stations.inBounds', [stations, bounds.getCenter()]);

		return stations;

	};

	/**
	 * Gibt Stationen im Radius "distance" und den Punkt "lat", "lng" zurück.
	 *
	 * @var float lat
	 * @var float lng
	 * @var int distance
	 * @return array
	 */
	function findNearestStation(lat, lng, distance) {

		var R = 6371,  // km
			P = Math.PI / 180,

			sin = Math.sin,
			cos = Math.cos,
			sqrt = Math.sqrt,

			dLat, dLng, lat_a, lat_b, a, c,

			filtered = [],

			station_number,
			station;

		for(station_number in stationData) {

			station = stationData[station_number];

			dLat = sin(((station.lat - lat) * P) / 2);
			dLng = sin(((station.lng - lng) * P) / 2);
			lat_a = cos(lat * P);
			lat_b = cos(station.lat * P);
			
			a = dLat * dLat + dLng * dLng * lat_a * lat_b;
			c = 2 * Math.atan2(sqrt(a), sqrt(1-a));

			station.distance = R * c;

			if(station.distance < distance) {
				filtered.push(station);
			}
		}

		if(filtered.length > 0) {

			filtered.sort(function(a, b) {

				return a.distance < b.distance ? -1 : a.distance > b.distance ? 1 : 0;

			});
		}

		return filtered;
	};

	/**
	 * Gibt eine Station anhand der Stationsnummer (XJ00000) zurück.
	 * Wird die Station nicht gefunden, wird ein leeres Objekt zurück gegeben.
	 *
	 * @var string station_id
	 * @return object
	 */
	function getStation(station_id) {

		if(stationData[station_id]) {
			return stationData[station_id];
		}

		return {};

	}

	/**
	 * Aktualisiert die Tankstellendaten und den User.
	 * Falls die Daten aktuell sind, wird der nur User aktualisert.
	 *
	 * @return jQuery.Deferred.promise
	 */
	function update() {

		var deferred = $.Deferred();

		// Zeit der angenommenen letzten Änderung auf dem Server (jeden Morgen um 4 Uhr).
		var targetDate = new Date();
		targetDate.setUTCDate(targetDate.getUTCDate()-1);
		targetDate.setUTCHours(4);
		targetDate.setUTCMinutes(0)
		targetDate.setUTCSeconds(0);
		targetDate.setUTCMilliseconds(0);

		if(lastUpdate > targetDate) {
			
			console.log('StationData is up to date');

			// Update User
			$.when(User.update()).then(function() {
				deferred.resolve();	
			}, function(jqXHR) {
				deferred.reject(jqXHR);
			});
			

		} else {

			console.log('Loading StationData');

			$.ajax({

	    		context: this,
	    		type : "POST",
	    		url: JET.backendURL,
	    		data: {
	    			action: 'getStationData',
	    			lastUpdate: lastUpdate,
	    			token: User.getToken()
	    		},
				dataType : "json",
				timeout : 10000,
				cache: false

	    	}).done(function(data) {

	    		if(data.status == 'success') {
	    			
	    			stationData = data.stations;
	    			lastUpdate = stationData.lastUpdate = (new Date()).getTime();

	    			window.localStorage['jet.app.stations'] = JSON.stringify(stationData);

	    			if(data.user) {
	    				User.setData(data.user);
	    			}

	    			deferred.resolve();

	    		} else {
	    			User.setData('');
	    			User.setToken('');
	    			deferred.reject();
	    		}

	    	}).fail(function(jqXHR) {

	    		// App ist offline
	    		if(isOfflineError(jqXHR)) {

	    			// Zeige Message, sobald die App "ready" ist.
					JET.one('app.ready', function() {
						displayNoConnectionError(true);
					});

	    		} else {

	    			User.setData('');
	    			User.setToken('');	

	    		}
	    		
	    		deferred.reject(jqXHR);

	    	}).always(function() {
	    		
	    		deferred.resolve();

	    	});

		}


		return deferred.promise();

	}

	/**
	 * Setzt einen Filter für die Suche der Tankstellen über getStationsInBounds
	 *
	 * @var string feature Name der Eigenschaft
	 * @var boolean status An / Aus
	 */
	function setFeature(feature, status) {

		features[feature] = status;

	}


	return {
		stations: function() {
			return stationData;
		},
		getStation: getStation,
		getStationsInBounds: findStationsInBounds,
		update: update,
		setFeature: setFeature,
		findNearestStation: findNearestStation
	};

})(jQuery);