/**
 * Diverse Hilfsfunktionen
 */

/**
 * Prüft, ob das Gerät eine Internetverbindung hat
 *
 * @returns {boolean}
 */
function isConnected() {
    if(!navigator.connection) return true;
    return !(navigator.connection.type === Connection.NONE);
}

/**
 * Prüft, ob ein jQuery Ajax Error an einer fehlerhaften Internetverbindung liegen kann.
 *
 * @param jqXHR
 * @returns {boolean}
 */
function isOfflineError(jqXHR) {

    if(typeof jqXHR === 'undefined') {
        return false;
    }

    if(jqXHR.status == 0) {
        return true;
    }

    return false;

}

/**
 * Zeigt eine Fehlermeldung an, die den User über die magelhafte Internetverbindung informiert.
 * Ist der Parameter isLoading === true, betrifft die Meldung einen Ladevorgang. Andernfalls wird von einem Speichervorgang ausgegangen
 *
 * @param isLoading
 */
function displayNoConnectionError(isLoading) {

    if(true !== isLoading) {
        isLoading = false;
    }

    var message;

    if(isLoading) {
        message = 'Die Daten können nicht geladen werden, da aktuell keine ausreichende Internetverbindung besteht.';
    } else {
        message = 'Die Aktion kann nicht korrekt ausgeführt werden, da aktuell keine ausreichende Internetverbindung besteht. Bitte versuche es später erneut.';
    }

    UI.alert('<b>Derzeit keine Internetverbindung!</b><br />' + message);

}

/**
 * Speichert den Pushtoken auf dem Server
 * Besteht keine ausreichende Internetverbindung, wird dies automatisch wiederholt, solbald die Verbindung wieder aufgebaut wurde.
 */
function savePushToken() {

	// Lösche ggf. vorhandenen Eventhandler
    $(document).off('online.savepushtoken');

	// Internet?
    if(!isConnected()) {
	    // Eventhandler, der diese Funktion erneut aufruft, sobald Internet wieder vorhanden ist.
        $(document).one('online.savepushtoken', savePushToken);
        return;
    }

	// API Parameter
    var params = {
        token: User.getToken()
    };

	// Je nach Gerät, muss ein anderer Token übergeben werden
    if(isAndroid()) {
        params.action = 'saveGCMToken';
        params.gcm = localStorage['gcmpushtoken'];
    } else {
        params.action = 'savePushToken';
        params.ios = localStorage['iospushtoken'];
    }

	// API Call
    $.post(JET.backendURL, params).fail(function(jqXHR) {
	    // Internet?
        if(isOfflineError(jqXHR) || !isConnected()) {
	        // Eventhandler, der diese Funktion erneut aufruft, sobald Internet wieder vorhanden ist.
            $(document).one('online.savepushtoken', savePushToken);
        }
    });

}

/**
 * Prüft eine E-Mail-Adresse auf formale Gültigkeit
 * @param email
 * @returns {boolean}
 */
function isValidEmail(email) {

	return /^.+@.+\..+$/.test(email);

}

/**
 * Prüft, ob das Gerät ein Android Gerät ist
 * @returns {boolean}
 */
function isAndroid() {

    if(typeof device == 'undefined') return false;

    return !!(device.platform.toLowerCase() == 'android');
}

/**
 * Wandelt einen Formularwert in ein float mit gegebener Anzahl an Nachkommastellen (Default 2)
 * @param input
 * @param precision
 * @returns {string}
 */
function string2number(input, precision) {

    if(input == '') return '';

    if(typeof precision == 'undefined') {
        precision = 2;
    }

    return parseFloat(input.toString().replace(/[^0-9,.]/, '').split(',').join('.')).toFixed(precision);

}

/**
 * Zeigt eine Liste mit Fehlermeldungen als erstes Element des übergebenen jQuery-Elements
 * @param errors Array mit Fehlermeldungen
 * @param el jQuery Element in dem die Meldungen ausgegeben werden sollen
 */
function showError(errors, el) {

	var errorString = ['<ul class="error">'];

	_.each(errors, function(value, key) {
		errorString.push('<li>'+value+'</li>');
		el.find(key).addClass('is-error');
	});

	errorString.push('</ul>');

	el.find('.error').remove()
	el.prepend(errorString.join(''));

}

/**
 * Zeigt eine Liste mit einer Infomeldung als erstes Element des übergebenen jQuery-Elements
 * @param message
 * @param el jQuery Element in dem die Meldung ausgegeben werden soll
 */
function showMessage(message, el) {

    var messageString = ['<ul class="info">'];

    messageString.push('<li>'+message+'</li>');

    messageString.push('</ul>');

    el.find('.info').remove();
    el.prepend(messageString.join(''));
}

/**
 * Zeigt eine Liste mit einer Erfolgsmeldung als erstes Element des übergebenen jQuery-Elements.
 * Die Meldung wird automatisch nach 5 Sekunden ausgeblendet
 * @param message
 * @param el jQuery Element in dem die Meldung ausgegeben werden soll
 */
function showSuccess(message, el) {
    var messageString = ['<ul class="success">'];

    messageString.push('<li>'+message+'</li>');

    messageString.push('</ul>');

    el.find('.success').remove();
    el.prepend($(messageString.join('')).delay(5000).fadeOut('slow'));
}

/**
 * Erstellt einen HTML Attribute String anhand eines Objects
 * Input: { title: 'Hallo', 'class': 'foo' }
 * Output: title="Hallo" class="foo"
 *
 * @param atts
 * @returns {string}
 */
function parseAttributes(atts) {
    var out = [];

    for(var attribute in atts) {

        out.push(' '+attribute+'="'+atts[attribute]+'"');

    }

    return out.join('');
}

/**
 * Erstellt ein öffnendes HTML-Tag
 * @param tag Name des Tags
 * @param atts string|object Attribute des Tags
 * @returns {string}
 */
function openTag(tag, atts) {

    var out = ['<'+tag];

    if(atts) {
        if(typeof atts == 'string') {
            out.push(' '+atts);
        } else {
            out.push(parseAttributes(atts));
        }
    }

    out.push('>');

    return out.join('');

}

/**
 * Erstellt ein schliessendes HTML-Tag
 * @param tag Name des Tags
 * @returns {string}
 */
function closeTag(tag) {
    return '</'+tag+'>';
}

/**
 * Bricht einen String nach einer bestimmten Anzahl Zeichen um (analog PHP wordwrap)
 *
 * @param str Die Eingabezeichenkette.
 * @param width Die Anzahl von Zeichen nach denen die Zeichenkette umgebrochen wird. (Default: 75)
 * @param brk Der optionale Parameter break wird als Zeichen(kette) für den Zeilenumbruch verwendet. (Default "\n")
 * @param cut Wenn der Parameter cut auf TRUE gesetzt ist, wird die Zeichenkette spätestens nach der angegebenen width umgebrochen.
 *            Wenn sie also ein Wort haben, das länger als die angegebene Maximallänge ist, wird der Umbruch im Wort selbst durchgeführt.
 *            Bei FALSE trennt die Funktion das Wort nicht, selbst wenn der width kleiner ist als die Wortlänge.
 * @returns string
 */
function wordwrap( str, width, brk, cut ) {
 
    brk = brk || '\n';
    width = width || 75;
    cut = cut || false;
 
    if (!str) { return str; }
 
    var regex = '.{1,' +width+ '}(\\s|$)' + (cut ? '|.{' +width+ '}|.+$' : '|\\S+?(\\s|$)');
 
    return str.match( RegExp(regex, 'g') ).join( brk );
 
}


/**************************************
 * Date class extension
 * 
 */
    // Provide month names
    Date.prototype.getMonthName = function(){
        var month_names = [
                            'Januar',
                            'Februar',
                            'März',
                            'April',
                            'Mai',
                            'Juni',
                            'Juli',
                            'August',
                            'September',
                            'Oktober',
                            'November',
                            'Dezember'
                        ];

        return month_names[this.getMonth()];
    }

    // Provide month abbreviation
    Date.prototype.getMonthAbbr = function(){
        var month_abbrs = [
                            'Jan',
                            'Feb',
                            'Mar',
                            'Apr',
                            'Mai',
                            'Jun',
                            'Jul',
                            'Aug',
                            'Sep',
                            'Okt',
                            'Nov',
                            'Dez'
                        ];

        return month_abbrs[this.getMonth()];
    }

    // Provide full day of week name
    Date.prototype.getDayFull = function(){
        var days_full = [
                            'Sonntag',
                            'Montag',
                            'Dienstag',
                            'Mittwoch',
                            'Donnerstag',
                            'Freitag',
                            'Samstag'
                        ];
        return days_full[this.getDay()];
    };

    // Provide full day of week name
    Date.prototype.getDayAbbr = function(){
        var days_abbr = [
                            'So',
                            'Mo',
                            'Di',
                            'Mi',
                            'Do',
                            'Fr',
                            'Sa'
                        ];
        return days_abbr[this.getDay()];
    };

    // Provide the day of year 1-365
    Date.prototype.getDayOfYear = function() {
        var onejan = new Date(this.getFullYear(),0,1);
        return Math.ceil((this - onejan) / 86400000);
    };

    // Provide the day suffix (st,nd,rd,th)
    Date.prototype.getDaySuffix = function() {
        var d = this.getDate();
        var sfx = ["th","st","nd","rd"];
        var val = d%100;

        return (sfx[(val-20)%10] || sfx[val] || sfx[0]);
    };

    // Provide Week of Year
    Date.prototype.getWeekOfYear = function() {
        var onejan = new Date(this.getFullYear(),0,1);
        return Math.ceil((((this - onejan) / 86400000) + onejan.getDay()+1)/7);
    } 

    // Provide if it is a leap year or not
    Date.prototype.isLeapYear = function(){
        var yr = this.getFullYear();

        if ((parseInt(yr)%4) == 0){
            if (parseInt(yr)%100 == 0){
                if (parseInt(yr)%400 != 0){
                    return false;
                }
                if (parseInt(yr)%400 == 0){
                    return true;
                }
            }
            if (parseInt(yr)%100 != 0){
                return true;
            }
        }
        if ((parseInt(yr)%4) != 0){
            return false;
        } 
    };

    // Provide Number of Days in a given month
    Date.prototype.getMonthDayCount = function() {
        var month_day_counts = [
                                    31,
                                    this.isLeapYear() ? 29 : 28,
                                    31,
                                    30,
                                    31,
                                    30,
                                    31,
                                    31,
                                    30,
                                    31,
                                    30,
                                    31
                                ];

        return month_day_counts[this.getMonth()];
    } 

    // format provided date into this.format format
    Date.prototype.format = function(dateFormat){
        // break apart format string into array of characters
        dateFormat = dateFormat.split("");

        var date = this.getDate(),
            month = this.getMonth(),
            hours = this.getHours(),
            minutes = this.getMinutes(),
            seconds = this.getSeconds();
        // get all date properties ( based on PHP date object functionality )
        var date_props = {
            d: date < 10 ? '0'+date : date,
            D: this.getDayAbbr(),
            j: this.getDate(),
            l: this.getDayFull(),
            S: this.getDaySuffix(),
            w: this.getDay(),
            z: this.getDayOfYear(),
            W: this.getWeekOfYear(),
            F: this.getMonthName(),
            m: month < 10 ? '0'+(month+1) : month+1,
            M: this.getMonthAbbr(),
            n: month+1,
            t: this.getMonthDayCount(),
            L: this.isLeapYear() ? '1' : '0',
            Y: this.getFullYear(),
            y: this.getFullYear()+''.substring(2,4),
            a: hours > 12 ? 'pm' : 'am',
            A: hours > 12 ? 'PM' : 'AM',
            g: hours % 12 > 0 ? hours % 12 : 12,
            G: hours > 0 ? hours : "12",
            h: hours % 12 > 0 ? hours % 12 : 12,
            H: hours < 10 ? '0'+hours : hours,
            i: minutes < 10 ? '0' + minutes : minutes,
            s: seconds < 10 ? '0' + seconds : seconds           
        };

        // loop through format array of characters and add matching data else add the format character (:,/, etc.)
        var date_string = "";
        for(var i=0;i<dateFormat.length;i++){
            var f = dateFormat[i];
            if(f.match(/[a-zA-Z]/g)){
                date_string += date_props[f] ? date_props[f] : '';
            } else {
                date_string += f;
            }
        }

        return date_string;
    };
/*
 *
 * END - Date class extension
 * 
 ************************************/


/**
 * Gibt ein Foto als data-URI zurück
 * Über den Parameter sourceType wird festgelegt, ob das Bild aus der Library oder aus der Camera kommt.
 *
 * @param sourceType
 * @returns jQuery.promise
 */
function getPhoto(sourceType) {

    var photoOptions = {
        quality: 50,
        destinationType: Camera.DestinationType.DATA_URL,
        sourceType: Camera.PictureSourceType[sourceType],
        encodingType: Camera.EncodingType.JPEG,
        allowEdit: true,
        targetWidth: 200,
        targetHeight:200,
        saveToPhotoAlbum: false,
        mediaType: Camera.MediaType.PICTURE,
        correctOrientation: true
    },
    dfr = $.Deferred();

    navigator.camera.getPicture(
        function(imageData) {

            dfr.resolve(imageData);

        },
        function() {

            dfr.reject();

        },
        photoOptions
    );

    return dfr.promise();

};
